<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;

class Capability extends Base
{
	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Fetch Capability',
			[self::argThreemaId, self::argFrom, self::argSecret],
			'Fetch the capabilities of a Threema ID'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$threemaId = $this->getArgumentThreemaId(self::argThreemaId);
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		Common::required($threemaId, $from, $secret);

		if (strlen($threemaId) != 8) {
			throw new Exception('invalid threema id');
		}
		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->keyCapability($threemaId);
		Common::required($result);
		if ($result->isSuccess()) {
			Common::l(implode("\n", $result->getCapabilities()));
		} else {
			Common::e($result->getErrorMessage());
		}
	}
}
