<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;

class Credits extends Base
{
	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Get remaining credits',
			[self::argFrom, self::argSecret],
			'Get the remaining credits'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		Common::required($from, $secret);

		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->credits();
		Common::required($result);
		if ($result->isSuccess()) {
			Common::l('remaining credits: ' . $result->getCredits());
		} else {
			Common::e($result->getErrorMessage());
		}
	}
}
