<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\Tools\CryptTool;

class DerivePublicKey extends Base
{
	public function __construct()
	{
		parent::__construct(
			'Derive Public Key',
			[self::argPrivateKey],
			'Derive the public key that corresponds with the given private key.'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);

		Common::required($privateKey);

		$cryptTool = CryptTool::getInstance();

		$publicKey = $cryptTool->derivePublicKey($privateKey);
		Common::l(Common::convertPublicKey(bin2hex($publicKey)));
	}
}
