<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use JsonException;
use Threema\Console\Common;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\Messages\TextMessage;
use Threema\MsgApi\Tools\CryptTool;

/**
 * Encrypt the stdin with the given @privateKey for the given @publicKey
 */
class Encrypt extends Base
{
	public function __construct()
	{
		parent::__construct(
			'Encrypt',
			[self::argPrivateKey, self::argPublicKey],
			'Encrypt standard input using the given sender private key and recipient public key. two lines to standard output: first the nonce (hex), and then the box (hex).'
		);
	}

	/**
	 * run the command
	 * @throws Exception
	 * @throws JsonException
	 */
	public function doRun()
	{
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$publicKey = $this->getArgumentPublicKey(self::argPublicKey);
		$textToEncrypt = $this->readStdIn();
		Common::required($privateKey, $publicKey, $textToEncrypt);

		$cryptTool = CryptTool::getInstance();
		//create a random nonce
		$newNonce = $cryptTool->randomNonce();
		$msg = new TextMessage($textToEncrypt);
		$encryptedMessageText = $cryptTool->encryptTextMessage($msg, $privateKey, $publicKey, $newNonce);

		Common::ln(bin2hex($newNonce));
		//output encrypted text
		Common::ln(bin2hex($encryptedMessageText));
	}
}
