<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\Tools\CryptTool;

class GenerateKeyPair extends Base
{
	public function __construct()
	{
		parent::__construct(
			'Generate Key Pair',
			[self::argPrivateKeyFile, self::argPublicKeyFile],
			'Generate a new key pair and write the private and public keys to the respective files (in hex).'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$keyPair = CryptTool::getInstance()->generateKeyPair();

		$privateKeyHex = bin2hex((string)$keyPair->privateKey);
		$publicKeyHex = bin2hex((string)$keyPair->publicKey);

		file_put_contents($this->getArgument(self::argPrivateKeyFile), Common::convertPrivateKey($privateKeyHex) . "\n");
		file_put_contents($this->getArgument(self::argPublicKeyFile), Common::convertPublicKey($publicKeyHex) . "\n");

		Common::l('key pair generated');
	}
}
