<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Exception;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupCreate extends Base
{
	final public const argMembers = 'members';

	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Send a End-to-End Encrypted Video Message',
			[self::argThreemaIds, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId],
			'Send a group create to the given ID (members are comma-separated group members, no members = remove group)',
			[self::argMembers]
		);
	}

	/**
	 * @throws \Threema\MsgApi\Core\Exception
	 * @throws Exception
	 */
	public function doRun()
	{
		$threemaIds = $this->getArgumentThreemaIds(self::argThreemaIds);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = trim($this->getArgument(self::argGroupId));

		if (strlen($groupId) != 8) {
			throw new Exception('invalid group id ' . $groupId);
		}

		$members = $this->getArgumentThreemaIds(self::argMembers) ?? [];
		$members[] = $from;

		Common::required($threemaIds, $from, $secret, $privateKey);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);
		$helper = new E2EHelper($privateKey, $connector);
		$results = $helper->sendGroupCreates(
			$threemaIds,
			$groupId,
			$members
		);
		foreach ($results->getResults() as $result) {
			if ($result->isSuccess()) {
				Common::l('Message ID: ' . $result->getMessageId());
			} else {
				Common::e('Error: ' . $result->getErrorMessage());
			}
		}
	}
}
