<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Exception;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupRename extends Base
{
	final public const argName = 'subject';

	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Rename a threema group',
			[self::argThreemaIds, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId, self::argName],
			'Rename a group rename to the given ID'
		);
	}

	/**
	 * @throws \Threema\MsgApi\Core\Exception
	 * @throws Exception
	 */
	public function doRun()
	{
		$threemaIds = $this->getArgumentThreemaIds(self::argThreemaIds);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = trim($this->getArgument(self::argGroupId));

		if (strlen($groupId) != 8) {
			throw new Exception('invalid group id ' . $groupId);
		}

		Common::required($threemaIds, $from, $secret, $privateKey);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);
		$helper = new E2EHelper($privateKey, $connector);

		$results = $helper->sendGroupRenames(
			$threemaIds,
			$groupId,
			$this->getArgument(self::argName)
		);

		foreach ($results->getResults() as $threemaId => $result) {
			if ($result->isSuccess()) {
				Common::l("{$threemaId} Message ID : {$result->getMessageId()}");
			} else {
				Common::e("{$threemaId} Error: {$result->getErrorMessage()}");
			}
		}
	}
}
