<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;

class LookupIdBulk extends Base
{
	final public const argEmailAddresses = 'emailAddresses';
	final public const argsPhoneNumbers = 'phoneNumbers';

	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Bulk ID-Lookup By Email Addresses And Phone Numbers',
			[self::argFrom, self::argSecret, self::argEmailAddresses, self::argsPhoneNumbers],
			'Lookup IDs linked to the given email or phone address (will be hashed locally).'
		);
	}

	/**
	 * @throws Exception
	 * @throws \Exception
	 */
	public function doRun()
	{
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		$emailAddresses = explode(',', (string)$this->getArgument(self::argEmailAddresses));
		$phoneNumbers = explode(',', (string)$this->getArgument(self::argsPhoneNumbers));

		Common::required($from, $secret);

		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->keyLookupBulk($emailAddresses, $phoneNumbers, false);
		Common::required($result);
		if ($result->isSuccess()) {
			foreach ($result->getResult() as $foundItem) {
				var_dump($foundItem);
			}
		} else {
			Common::e($result->getErrorMessage());
		}
	}
}
