<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;

class LookupIdByEmail extends Base
{
	final public const argEmail = 'email';

	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'ID-Lookup By Email Address',
			[self::argEmail, self::argFrom, self::argSecret],
			'Lookup the ID linked to the given email address (will be hashed locally).'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$email = $this->getArgument(self::argEmail);
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		Common::required($email, $from, $secret);

		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->keyLookupByEmail($email);
		Common::required($result);
		if ($result->isSuccess()) {
			Common::l($result->getId());
		} else {
			Common::e($result->getErrorMessage());
		}
	}
}
