<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;

class LookupPublicKeyById extends Base
{
	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Fetch Public Key',
			[self::argThreemaId, self::argFrom, self::argSecret],
			'Lookup the public key for the given ID.'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$id = $this->getArgumentThreemaId(self::argThreemaId);
		$from = $this->getArgumentThreemaId(self::argFrom);
		$secret = $this->getArgument(self::argSecret);

		Common::required($id, $from, $secret);

		//define connection settings
		$settings = new ConnectionSettings($from, $secret);

		//create a connection
		$connector = new Connection($settings, $this->publicKeyStore);

		$result = $connector->fetchPublicKey($id);
		if ($result->isSuccess()) {
			Common::l(Common::convertPublicKey($result->getPublicKey()));
		} else {
			Common::e($result->getErrorMessage());
		}
	}
}
