<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class SendE2EVideo extends Base
{
	final public const argDuration = 'duration';
	final public const argVideoFile = 'videoFile';
	final public const argThumbnailFile = 'thumbnailFile';

	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'[DEPRECATED] Send a End-to-End Encrypted Video Message',
			[self::argThreemaId, self::argFrom, self::argSecret, self::argPrivateKey, self::argDuration, self::argVideoFile, self::argThumbnailFile],
			'Encrypt the video and thumbnail file and send the message to the given ID. \'from\' is the API identity and \'secret\' is the API secret. Prints the message ID on success.'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$threemaId = $this->getArgument(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);

		$videoPath = $this->getArgumentFile(self::argVideoFile);
		$duration = $this->getArgument(self::argDuration);
		$thumbnailPath = $this->getArgumentFile(self::argThumbnailFile);

		Common::required($threemaId, $from, $secret, $privateKey, $videoPath, $duration);

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);

		$helper = new E2EHelper($privateKey, $connector);
		$result = $helper->sendVideoMessage(
			$threemaId,
			$duration,
			$videoPath,
			$thumbnailPath
		);

		if ($result->isSuccess()) {
			Common::l('Message ID: ' . $result->getMessageId());
		} else {
			Common::e('Error: ' . $result->getErrorMessage());
		}
	}
}
