<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use InvalidArgumentException;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\PublicKeyStore;
use Threema\MsgApi\Receiver;

class SendSimple extends Base
{
	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Send Simple Message',
			[self::argThreemaId, self::argFrom, self::argSecret],
			'Send a message from standard input with server-side encryption to the given ID. \'from\' is the API identity and \'secret\' is the API secret. Prints the message ID on success.'
		);
	}

	/**
	 * @throws Exception
	 */
	public function doRun()
	{
		$to = $this->getArgument(self::argThreemaId);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		Common::required($to, $from, $secret);

		$message = $this->readStdIn();
		if (strlen($message) === 0) {
			throw new InvalidArgumentException('please define a message');
		}

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);
		$receiver = new Receiver($to, Receiver::TYPE_ID);

		$result = $connector->sendSimple($receiver, $message);
		if ($result->isSuccess()) {
			Common::l('Message ID: ' . $result->getMessageId());
		} else {
			Common::l('Error: ' . $result->getErrorMessage());
		}
	}
}
