<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands;

use Threema\MsgApi\Commands\Results\LookupBulkIdResult;
use Threema\MsgApi\Tools\CryptTool;

class LookupIdBulk implements JsonCommandInterface
{
	private readonly bool $hashed;

	public function __construct(private readonly array $emailAddresses, private readonly array $phoneNumbers, bool $hashed = true)
	{
		$this->hashed = true === $hashed;
	}

	public function getData(): array
	{
		$emailHashes = $this->emailAddresses;
		$phoneHashes = $this->phoneNumbers;

		if (false === $this->hashed) {
			//hash all email and phone numbers
			$cryptoTool = CryptTool::getInstance();
			foreach ($emailHashes as $index => $email) {
				$emailHashes[$index] = $cryptoTool->hashEmail($email);
			}
			foreach ($phoneHashes as $index => $phone) {
				$phoneHashes[$index] = $cryptoTool->hashPhoneNo($phone);
			}
		}

		return [
			'phoneHashes' => $phoneHashes,
			'emailHashes' => $emailHashes
		];
	}

	public function getParams(): array
	{
		return [];
	}

	public function getPath(): string
	{
		return 'lookup/bulk';
	}

	public function parseResult(int $httpCode, string $res): LookupBulkIdResult
	{
		return new LookupBulkIdResult($httpCode, $res);
	}
}
