<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands\Results;

class CapabilityResult extends Result
{
	/** @var string[] */
	private array $capabilities;

	private function can(mixed $key): bool
	{
		return true === in_array($key, $this->capabilities);
	}

	protected function processResponse(?string $response): void
	{
		if (null === $response) {
			$this->capabilities = [];

			return;
		}
		$this->capabilities =
			array_unique(array_filter(explode(',', strlen($response) > 0 ? $response : '')));
	}

	protected function getErrorMessageByErrorCode(int $httpCode): string
	{
		return match ($httpCode) {
			401 => 'API identity or secret incorrect',
			404 => 'No matching ID found',
			500 => 'A temporary internal server error has occurred',
			default => 'Unknown error',
		};
	}

	/**
	 * @return string[]
	 */
	public function getCapabilities(): array
	{
		return $this->capabilities;
	}

	/**
	 * the threema id can receive text
	 */
	public function canText(): bool
	{
		return $this->can('text');
	}

	/**
	 * the threema id can receive images
	 */
	public function canImage(): bool
	{
		return $this->can('image');
	}

	/**
	 * the threema id can receive videos
	 */
	public function canVideo(): bool
	{
		return $this->can('video');
	}

	/**
	 * the threema id can receive files
	 */
	public function canAudio(): bool
	{
		return $this->can('audio');
	}

	/**
	 * the threema id can receive files
	 */
	public function canFile(): bool
	{
		return $this->can('file');
	}

	/**
	 * the threema id can receive group messages
	 */
	public function canGroup(): bool
	{
		return $this->can('group');
	}

	/**
	 * the threema id can receive ballot messages
	 */
	public function canBallot(): bool
	{
		return $this->can('ballot');
	}

	/**
	 * the threema id can receive calls
	 */
	public function canCall(): bool
	{
		return $this->can('call');
	}

	/**
	 * the threema id can receive video calls
	 */
	public function canVideoCall(): bool
	{
		return $this->can('videocall');
	}

	/**
	 * the threema id can receive pfs messages
	 */
	public function canPfs(): bool
	{
		return $this->can('pfs');
	}

	/**
	 * the threema id can receive group calls
	 */
	public function canGroupCall(): bool
	{
		return $this->can('groupcall');
	}
}
