<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands\Results;

use LookupBulkIdResultItem;

class LookupBulkIdResult extends Result
{
	/** @var LookupBulkIdResultItem[] */
	private array $result = [];

	protected function processResponse(?string $response): void
	{
		try {
			$this->result = json_decode((string)$response, true, flags: JSON_THROW_ON_ERROR);
		} catch (\JsonException) {
		}
	}

	protected function getErrorMessageByErrorCode(int $httpCode): string
	{
		return match ($httpCode) {
			400 => 'Hash length is wrong',
			401 => 'API identity or secret incorrect',
			404 => 'No matching ID found',
			413 => 'Too many hashes have been specified in the request',
			500 => 'A temporary internal server error has occurred',
			default => 'Unknown error',
		};
	}

	/**
	 * @return LookupBulkIdResultItem[]
	 */
	public function getResult(): array
	{
		return $this->result;
	}
}
