<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands\Results;

abstract class Result
{
	private ?string $response;

	public function __construct(protected int $httpCode, ?string $response)
	{
		$this->processResponse($response);
		$this->response = $response;
	}

	abstract protected function getErrorMessageByErrorCode(int $httpCode): string;

	abstract protected function processResponse(?string $response): void;

	public function isSuccess(): bool
	{
		return 200 == $this->httpCode;
	}

	final public function getErrorCode(): ?int
	{
		if (false === $this->isSuccess()) {
			return $this->httpCode;
		}

		return null;
	}

	final public function getErrorMessage(): string
	{
		return $this->getErrorMessageByErrorCode($this->getErrorCode());
	}

	final public function getRawResponse(): string
	{
		return $this->response;
	}
}
