<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands\Results;

class SendE2EBulkResult extends Result
{
	/** @var SendE2EResult[] */
	private array $results = [];

	protected function processResponse(?string $response): void
	{
		$r = json_decode((string)$response, true);

		if (!is_array($r)) {
			//should not happen!
			return;
		}

		foreach ($r as $res) {
			$httpCode = $res['errorCode'] ?? $this->httpCode;
			$this->results[] = new SendE2EResult($httpCode, $res['messageId'] ?? null);
		}
	}

	protected function getErrorMessageByErrorCode(int $httpCode): string
	{
		return match ($httpCode) {
			400 => 'The recipient identity is invalid or the account is not set up for E2E mode',
			401 => 'API identity or secret incorrect',
			402 => 'No credits remain',
			413 => 'Message is too long',
			500 => 'A temporary internal server error has occurred',
			default => 'Unknown error',
		};
	}

	/**
	 * @return SendE2EResult[]
	 */
	public function getResults(): array
	{
		return $this->results;
	}
}
