<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands;

use Threema\MsgApi\Commands\Results\SendE2EResult;

class SendE2E implements CommandInterface
{
	private readonly array $moreOptions;

	public function __construct(
		private readonly string $threemaId,
		private readonly string $nonce,
		private readonly string $box,
		?array $moreOptions = null
	) {
		$this->moreOptions = $moreOptions ?? [];
	}

	public function getNonce(): string
	{
		return $this->nonce;
	}

	public function getBox(): string
	{
		return $this->box;
	}

	public function getParams(): array
	{
		$p = [];
		$p['to'] = $this->threemaId;
		$p['nonce'] = bin2hex($this->getNonce());
		$p['box'] = bin2hex($this->getBox());

		return array_merge($this->moreOptions, $p);
	}

	public function getPath(): string
	{
		return 'send_e2e';
	}

	public function parseResult(int $httpCode, string $res): SendE2EResult
	{
		return new SendE2EResult($httpCode, $res);
	}
}
