<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Commands;

use Threema\MsgApi\Commands\Results\SendE2EBulkResult;

class SendE2EBulk implements JsonCommandInterface
{
	/** @var SendE2E[] */
	private array $commands = [];

	private readonly array $params;

	public function __construct(?array $params = null)
	{
		$this->params = $params ?? [];
	}

	/**
	 * @return $this
	 */
	public function addCommand(SendE2E $command): self
	{
		$this->commands[] = $command;

		return $this;
	}

	public function getData(): array
	{
		$data = [];
		foreach ($this->commands as $cmd) {
			$p = array_merge($cmd->getParams());
			$p['nonce'] = base64_encode($cmd->getNonce());
			$p['box'] = base64_encode($cmd->getBox());
			$data[] = $p;
		}

		return $data;
	}

	public function getParams(): array
	{
		return $this->params;
	}

	public function getPath(): string
	{
		return 'send_e2e_bulk';
	}

	public function parseResult(int $httpCode, string $res): SendE2EBulkResult
	{
		return new SendE2EBulkResult($httpCode, $res);
	}
}
