<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en
 */

namespace Threema\MsgApi\Core;

use JsonException;

class AssocArray
{
	private readonly array $data;

	public function __construct(array $data)
	{
		//be sure a array is set
		$this->data = $data ?? [];
	}

	/**
	 * @param null $defaultValue
	 * @return mixed|null return the key value or the default value
	 */
	public function getValue(string $key, $defaultValue = null)
	{
		if (false === array_key_exists($key, $this->data)) {
			return $defaultValue;
		}

		return $this->data[$key];
	}

	/**
	 * @param ?array $requiredKeys
	 * @param mixed $string
	 * @throws Exception
	 * @return AssocArray
	 */
	final public static function byJsonString($string, ?array $requiredKeys = null)
	{
		try {
			$v = json_decode((string)$string, true, 512, JSON_THROW_ON_ERROR);
		} catch (JsonException $e) {
			throw new Exception('invalid json string');
		}
//		if (null === $v || false === $v) {
//			throw new Exception('invalid json string');
//		}

		return self::byAssocArray($v, $requiredKeys);
	}

	/**
	 * @param ?array $requiredKeys
	 * @throws Exception
	 * @return AssocArray
	 */
	final public static function byAssocArray(array $array, ?array $requiredKeys = null)
	{
		//validate first
		if (null !== $requiredKeys) {
			//validate array first
			foreach ($requiredKeys as $requiredKey) {
				if (false === array_key_exists($requiredKey, $array)) {
					throw new Exception('required key ' . $requiredKey . ' failed');
				}
			}
		}

		return new AssocArray($array);
	}
}
