<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Helpers;

use Threema\MsgApi\Messages\ThreemaMessage;

class ReceiveMessageResult
{
	/** @var string[] */
	private array $files = [];

	/** @var string[] */
	private array $errors = [];

	public function __construct(private readonly string $messageId, private readonly ThreemaMessage $threemaMessage)
	{
	}

	public function getMessageId(): string
	{
		return $this->messageId;
	}

	/**
	 * @return $this
	 */
	public function addError(string $message): static
	{
		$this->errors[] = $message;

		return $this;
	}

	public function isSuccess(): bool
	{
		return null === $this->errors || count($this->errors) == 0;
	}

	/**
	 * @return $this
	 */
	public function addFile(string $key, string $file): static
	{
		$this->files[$key] = $file;

		return $this;
	}

	/**
	 * @return \string[]
	 */
	public function getErrors(): array
	{
		return $this->errors;
	}

	public function getThreemaMessage(): ThreemaMessage
	{
		return $this->threemaMessage;
	}

	/**
	 * @return \string[]
	 */
	public function getFiles(): array
	{
		return $this->files;
	}
}
