<?php
/**
 * FileMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Exception;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\FileRenderingType;

/**
 * FileMessage Class Doc Comment
 *
 * @category Class
 * @description A File Message
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class FileMessage extends ThreemaMessage
{
	public const TYPE_CODE = 0x17;

	public function __construct(
		private readonly string $mimeType,
		private readonly int $size,
		private readonly string $blobId,
		private readonly string $encryptionKey,
		private readonly ?string $thumbnailBlobId = null,
		private readonly ?string $thumbnailMediaType = null,
		private readonly ?string $caption = null,
		private readonly ?FileRenderingType $fileRenderingType = FileRenderingType::FILE,
		private readonly ?string $correlationId = null,
		private readonly ?array $metadata = null,
		private readonly ?string $fileName = 'unnamed'
	) {
	}

	public function __toArray(): array
	{
		return array_filter([
			'm' => $this->getMimeType(),
			's' => $this->getSize(),
			'b' => $this->getBlobId(),
			'k' => bin2hex($this->getEncryptionKey()),
			't' => $this->getThumbnailBlobId(),
			'p' => $this->getThumbnailMediaType(),
			'd' => $this->getCaption(),
			'j' => null !== $this->getRenderingType() ? $this->getRenderingType()->value : throw new \Threema\MsgApi\Exceptions\InvalidArgumentException('unhandled rendering type'),
			'c' => $this->getCorrelationId(),
			'x' => $this->getMetadata(),
			'n' => $this->getFilename()
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'FileMessage{' . 'mimeType=' . $this->mimeType . ', '
			. 'size=' . $this->size . ', '
			. 'blobId=' . $this->blobId . ', '
			. 'encryptionKey=' . $this->encryptionKey . ', '
			. 'thumbnailBlobId=' . $this->thumbnailBlobId . ', '
			. 'thumbnailMediaType=' . $this->thumbnailMediaType . ', '
			. 'caption=' . $this->caption . ', '
			. 'fileRenderingType=' . $this->fileRenderingType->name . ', '
			. 'correlationId=' . $this->correlationId . ', '
			. 'metadata=' . print_r($this->metadata, true) . ', '
			. 'fileName=' . $this->fileName . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized FileMessage.
	 *
	 * @throws Exception
	 */
	public static function fromString(string $data, int $realDataLength, Closure $piece): FileMessage
	{
		Serializers\FileMessageSerializer::validate($data, $realDataLength);

		return Serializers\FileMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->mimeType) {
			$invalidProperties[] = "'mimeType' can't be null";
		}
		if (null === $this->size) {
			$invalidProperties[] = "'size' can't be null";
		}
		if (null === $this->blobId) {
			$invalidProperties[] = "'blobId' can't be null";
		}
		if (null === $this->encryptionKey) {
			$invalidProperties[] = "'encryptionKey' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets mimeType
	 */
	public function getMimeType(): string
	{
		return $this->mimeType;
	}

	/**
	 * Gets size
	 */
	public function getSize(): int
	{
		return $this->size;
	}

	/**
	 * Gets blobId
	 */
	public function getBlobId(): string
	{
		return $this->blobId;
	}

	/**
	 * Gets encryptionKey
	 */
	public function getEncryptionKey(): string
	{
		return $this->encryptionKey;
	}

	/**
	 * Gets thumbnailBlobId
	 */
	public function getThumbnailBlobId(): ?string
	{
		return $this->thumbnailBlobId;
	}

	/**
	 * Gets thumbnailMediaType
	 */
	public function getThumbnailMediaType(): ?string
	{
		return $this->thumbnailMediaType;
	}

	/**
	 * Gets caption
	 */
	public function getCaption(): ?string
	{
		return $this->caption;
	}

	/**
	 * Gets fileRenderingType
	 */
	public function getRenderingType(): ?FileRenderingType
	{
		return $this->fileRenderingType;
	}

	/**
	 * Gets correlationId
	 */
	public function getCorrelationId(): ?string
	{
		return $this->correlationId;
	}

	/**
	 * Gets metadata
	 */
	public function getMetadata(): ?array
	{
		return $this->metadata;
	}

	/**
	 * Gets fileName
	 */
	public function getFilename(): ?string
	{
		return $this->fileName;
	}
}
