<?php
/**
 * GroupBallotCreateMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Exception;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\Voting\BallotChoice;
use Threema\MsgApi\Types\Voting\DisplayMode;
use Threema\MsgApi\Types\Voting\ResultsDisclosureType;
use Threema\MsgApi\Types\Voting\State;
use Threema\MsgApi\Types\Voting\VotingMode;

/**
 * GroupBallotCreateMessage Class Doc Comment
 *
 * @category Class
 * @description A group poll create message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class GroupBallotCreateMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x52;

	public function __construct(
		private readonly GroupId $groupId,
		private readonly BallotId $ballotId,
		private readonly string $description,
		private readonly State $state,
		private readonly VotingMode $votingMode,
		private readonly ResultsDisclosureType $resultsDisclosureType,
		private readonly DisplayMode $displayMode,
		private readonly array $choices,
		private readonly ?array $participants = null
	) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([
			'd' => $this->getDescription(),
			's' => $this->getState()->value,
			'a' => $this->getVotingMode()->value,
			't' => $this->getResultsDisclosureType()->value,
			'o' => 0,
			'u' => $this->getDisplayMode()->value,
			'c' => array_values(
				array_map(
					fn (BallotChoice $choices) => array_filter($choices->__toArray(), fn ($v) => Util::array_filter_callback($v)),
					$this->getChoices()
				)
			),
			'p' => $this->getParticipants()
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'GroupBallotCreateMessage{' . 'groupId=' . $this->groupId . ', '
			. 'ballotId=' . $this->ballotId . ', '
			. 'description=' . $this->description . ', '
			. 'state=' . $this->state->name . ', '
			. 'votingMode=' . $this->votingMode->name . ', '
			. 'resultsDisclosureType=' . $this->resultsDisclosureType->name . ', '
			. 'displayMode=' . $this->displayMode->name . ', '
			. 'choices=' . print_r($this->choices, true) . ', '
			. 'participants=' . print_r($this->participants, true) . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupBallotCreateMessage.
	 *
	 * @throws Exception
	 */
	public static function fromString(string $data, int $realDataLength, Closure $piece): GroupBallotCreateMessage
	{
		Serializers\GroupBallotCreateMessageSerializer::validate($data, $realDataLength);

		return Serializers\GroupBallotCreateMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->ballotId) {
			$invalidProperties[] = "'ballotId' can't be null";
		}
		if (null === $this->description) {
			$invalidProperties[] = "'description' can't be null";
		}
		if (null === $this->state) {
			$invalidProperties[] = "'state' can't be null";
		}
		if (null === $this->votingMode) {
			$invalidProperties[] = "'votingMode' can't be null";
		}
		if (null === $this->resultsDisclosureType) {
			$invalidProperties[] = "'resultsDisclosureType' can't be null";
		}
		if (null === $this->displayMode) {
			$invalidProperties[] = "'displayMode' can't be null";
		}
		if (null === $this->choices) {
			$invalidProperties[] = "'choices' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets groupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets ballotId
	 */
	public function getBallotId(): BallotId
	{
		return $this->ballotId;
	}

	/**
	 * Gets description
	 */
	public function getDescription(): string
	{
		return $this->description;
	}

	/**
	 * Gets state
	 */
	public function getState(): State
	{
		return $this->state;
	}

	/**
	 * Gets votingMode
	 */
	public function getVotingMode(): VotingMode
	{
		return $this->votingMode;
	}

	/**
	 * Gets resultsDisclosureType
	 */
	public function getResultsDisclosureType(): ResultsDisclosureType
	{
		return $this->resultsDisclosureType;
	}

	/**
	 * Gets displayMode
	 */
	public function getDisplayMode(): DisplayMode
	{
		return $this->displayMode;
	}

	/**
	 * Gets choices
	 */
	public function getChoices(): array
	{
		return $this->choices;
	}

	/**
	 * Gets participants
	 */
	public function getParticipants(): ?array
	{
		return $this->participants;
	}
}
