<?php
/**
 * GroupBallotVoteMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;

/**
 * GroupBallotVoteMessage Class Doc Comment
 *
 * @category Class
 * @description A group poll vote message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see     https://gateway.threema.ch/en/developer
 */
class GroupBallotVoteMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x53;

	public function __construct(
		private readonly GroupId $groupId,
		private readonly string $creator,
		private readonly BallotId $ballotId,
		private readonly array $votes
	) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return $this->getVotes();
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'GroupBallotVoteMessage{' . 'groupId=' . $this->groupId . ', '
			. 'creator=' . $this->creator . ', '
			. 'ballotId=' . $this->ballotId . ', '
			. 'votes=' . print_r($this->votes, true) . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupBallotVoteMessage.
	 *
	 * @throws BadMessageException
	 */
	public static function fromString(string $data, int $realDataLength, Closure $piece): GroupBallotVoteMessage
	{
		Serializers\GroupBallotVoteMessageSerializer::validate($data, $realDataLength);

		return Serializers\GroupBallotVoteMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->creator) {
			$invalidProperties[] = "'creator' can't be null";
		}
		if (null === $this->ballotId) {
			$invalidProperties[] = "'ballotId' can't be null";
		}
		if (null === $this->votes) {
			$invalidProperties[] = "'votes' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets groupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets creator
	 */
	public function getCreator(): string
	{
		return $this->creator;
	}

	/**
	 * Gets ballotId
	 */
	public function getBallotId(): BallotId
	{
		return $this->ballotId;
	}

	/**
	 * Gets votes
	 */
	public function getVotes(): array
	{
		return $this->votes;
	}
}
