<?php
/**
 * GroupDeliveryReceipt
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Exception;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\DeliveryReceiptType;
use Threema\MsgApi\Types\GroupId;

/**
 * GroupDeliveryReceipt Class Doc Comment
 *
 * @category Class
 * @description A group delivery receipt message that can be sent/received with end-to-end encryption via Threema. Each group delivery receipt message confirms the receipt of one or multiple group messages.
 * @author   Threema Gateway team
 * @see      https://gateway.threema.ch/en/developer
 */
class GroupDeliveryReceipt extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x81;

	public function __construct(
		private readonly GroupId $groupId,
		private readonly DeliveryReceiptType $receiptType,
		private readonly array $ackedMessageIds
	) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		$str = 'GroupDeliveryReceipt (' . 'groupId=' . $this->groupId . ', '
			. $this->getReceiptType()->name . '): ';

		$hexAckedMessageIds = [];
		foreach ($this->ackedMessageIds as $ackedMessageIdsItem) {
			$hexAckedMessageIds[] = bin2hex((string)$ackedMessageIdsItem);
		}
		$str .= join(', ', $hexAckedMessageIds);

		return $str;
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		$binMessageIds = [];
		foreach ($this->getAckedMessageIds() as $msgId) {
			$binMessageIds[] = hex2bin($msgId);
		}
		return chr($this->getReceiptType()->value) . implode($binMessageIds);
	}

	/**
	 * The deserialized GroupDeliveryReceipt.
	 *
	 * @throws Exception
	 */
	public static function fromString(string $data, int $realDataLength, Closure $piece): GroupDeliveryReceipt
	{
		Serializers\DeliveryReceiptSerializer::validate($data, $realDataLength);

		return Serializers\GroupDeliveryReceiptSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->receiptType) {
			$invalidProperties[] = "'receiptType' can't be null";
		}
		if (null === $this->ackedMessageIds) {
			$invalidProperties[] = "'ackedMessageIds' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets groupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets receiptType
	 */
	public function getReceiptType(): DeliveryReceiptType
	{
		return $this->receiptType;
	}

	/**
	 * Gets ackedMessageIds
	 */
	public function getAckedMessageIds(): array
	{
		return $this->ackedMessageIds;
	}
}
