<?php
/**
 * GroupLocationMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Closure;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\GroupId;

/**
 * GroupLocationMessage Class Doc Comment
 *
 * @category Class
 * @description A Location Message
 * @author   Threema Gateway team
 * @see     https://gateway.threema.ch/en/developer
 */
class GroupLocationMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x42;

	public function __construct(
		private readonly GroupId $groupId,
		private readonly string $latitude,
		private readonly string $longitude,
		private readonly ?string $address,
		private readonly ?float $accuracy,
		private readonly ?string $poiName
	) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'GroupLocationMessage{' . 'groupId=' . $this->groupId . ', '
			. 'latitude=' . $this->latitude . ', '
			. 'longitude=' . $this->longitude . ', '
			. 'address=' . $this->address . ', '
			. 'accuracy=' . $this->accuracy . ', '
			. 'poiName=' . $this->poiName . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupLocationMessage.
	 *
	 * @throws BadMessageException
	 */
	public static function fromString(string $data, int $realDataLength, Closure $piece): GroupLocationMessage
	{
		Serializers\GroupLocationMessageSerializer::validate($data, $realDataLength);

		return Serializers\GroupLocationMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->latitude) {
			$invalidProperties[] = "'latitude' can't be null";
		}
		if (null === $this->longitude) {
			$invalidProperties[] = "'longitude' can't be null";
		}
		if (null === $this->address) {
			$invalidProperties[] = "'address' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets groupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets latitude
	 */
	public function getLatitude(): string
	{
		return $this->latitude;
	}

	/**
	 * Gets longitude
	 */
	public function getLongitude(): string
	{
		return $this->longitude;
	}

	/**
	 * Gets address
	 */
	public function getPoiAddress(): ?string
	{
		return $this->address;
	}

	/**
	 * Gets accuracy
	 */
	public function getAccuracy(): ?float
	{
		return $this->accuracy;
	}

	/**
	 * Gets poiName
	 */
	public function getPoiName(): ?string
	{
		return $this->poiName;
	}
}
