<?php
/**
 * GroupTextMessage
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Messages;

use Threema\MsgApi\Exceptions\UnsupportedMessageTypeException;
use Threema\MsgApi\Serializers;
use Threema\MsgApi\Tools\Util;
use Threema\MsgApi\Types\GroupId;

/**
 * GroupTextMessage Class Doc Comment
 *
 * @category Class
 * @description A group text message that can be sent/received with end-to-end encryption via Threema.
 * @author   Threema Gateway team
 * @see     https://gateway.threema.ch/en/developer
 */
class GroupTextMessage extends ThreemaGroupMessage
{
	public const TYPE_CODE = 0x41;

	public function __construct(
		private readonly GroupId $groupId,
		private readonly string $text
	) {
		parent::__construct($groupId);
	}

	public function __toArray(): array
	{
		return array_filter([
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'GroupTextMessage{' . 'groupId=' . $this->groupId . ', '
			. 'text=' . $this->text . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * The deserialized GroupTextMessage.
	 *
	 * @param mixed $piece
	 * @throws UnsupportedMessageTypeException
	 */
	public static function fromString(string $data, int $realDataLength, $piece): GroupTextMessage
	{
		Serializers\GroupTextMessageSerializer::validate($data, $realDataLength);

		return Serializers\GroupTextMessageSerializer::deserialize($data, $piece);
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = parent::listInvalidProperties();

		if (null === $this->groupId) {
			$invalidProperties[] = "'groupId' can't be null";
		}
		if (null === $this->text) {
			$invalidProperties[] = "'text' can't be null";
		}

		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Get the message type code of this message.
	 *
	 * @return int message type code
	 */
	public function getTypeCode(): int
	{
		return self::TYPE_CODE;
	}

	/**
	 * Gets groupId
	 */
	public function getGroupId(): GroupId
	{
		return $this->groupId;
	}

	/**
	 * Gets text
	 */
	public function getText(): string
	{
		return $this->text;
	}

	/**
	 * Whether or not the message is a quote
	 */
	public function isQuote(): bool
	{
		return preg_match('/^> quote #[[:alnum:]]{16}\n\n.+$/', $this->text);
	}

	/**
	 * Gets quoted message id if the message is a quote
	 */
	public function getQuotedMessageId(): ?string
	{
		if (false === $this->isQuote()) {
			return null;
		}
		$matches = [];
		preg_match('/^> quote #([[:alnum:]]{16})\n\n.+$/', $this->text, $matches, PREG_UNMATCHED_AS_NULL);

		if (array_key_exists(1, $matches) && !empty($matches[1])) {
			return $matches[1];
		}

		return null;
	}
}
