<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi;

/**
 * Interface PublicKeyStore
 * Store the fetched Public Keys
 */
abstract class PublicKeyStore
{
	/** threemaId => publicKey cache */
	private array $cache = [];

	/**
	 * return null if the public key not found in the store
	 */
	abstract protected function findPublicKey(string $threemaId): ?string;

	/**
	 * save a public key
	 */
	abstract protected function savePublicKey(string $threemaId, string $publicKey): bool;

	/**
	 * return null if the public key not found in the store
	 */
	final public function getPublicKey(string $threemaId): ?string
	{
		if (array_key_exists($threemaId, $this->cache)) {
			return $this->cache[$threemaId];
		}

		$publicKey = $this->findPublicKey($threemaId);
		if (null !== $publicKey) {
			$this->cache[$threemaId] = $publicKey;
		}

		return $publicKey;
	}

	/**
	 * set and save a public key
	 */
	final public function setPublicKey(string $threemaId, string $publicKey): bool
	{
		$this->cache[$threemaId] = $publicKey;

		return $this->savePublicKey($threemaId, $publicKey);
	}
}
