<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi;

use InvalidArgumentException;

class Receiver
{
	final public const TYPE_ID = 'to';
	final public const TYPE_PHONE = 'phone';
	final public const TYPE_EMAIL = 'email';

	private string $type = self::TYPE_ID;
	private string $value;

	public function __construct(string $value, string $type = self::TYPE_ID)
	{
		$this->setValue($value, $type);
	}

	/**
	 * @return $this
	 */
	private function setValue(string $value, string $type)
	{
		$this->value = $value;
		$this->type = $type;

		return $this;
	}

	/**
	 * @return $this
	 */
	public function setToThreemaId(string $threemaId)
	{
		return $this->setValue(
			$threemaId,
			self::TYPE_ID
		);
	}

	/**
	 * @return $this
	 */
	public function setToPhoneNo(string $phoneNo)
	{
		return $this->setValue(
			$phoneNo,
			self::TYPE_PHONE
		);
	}

	/**
	 * @return $this
	 */
	public function setToEmail(string $emailAddress)
	{
		return $this->setValue(
			$emailAddress,
			self::TYPE_EMAIL
		);
	}

	/**
	 * @throws InvalidArgumentException
	 * @return array
	 */
	public function getParams()
	{
		switch ($this->type) {
			case self::TYPE_ID:
				$to = $this->type;
				$this->value = strtoupper(trim($this->value));

				break;

			case self::TYPE_EMAIL:
			case self::TYPE_PHONE:
				$to = $this->type;

				break;
			default:
				throw new InvalidArgumentException();
		}

		return [
			$to => $this->value
		];
	}
}
