<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Core\AssocArray;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Exceptions\InvalidArgumentException;
use Threema\MsgApi\Messages\BallotCreateMessage;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\Voting\BallotChoice;
use Threema\MsgApi\Types\Voting\DisplayMode;
use Threema\MsgApi\Types\Voting\ResultsDisclosureType;
use Threema\MsgApi\Types\Voting\State;
use Threema\MsgApi\Types\Voting\VotingMode;

class BallotCreateMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws BadMessageException
	 * @throws InvalidArgumentException
	 * @throws \Threema\MsgApi\Core\Exception
	 */
	public static function deserialize(string $data, Closure $piece): BallotCreateMessage
	{
		$ballotId = new BallotId($piece->__invoke(Constants::BALLOT_ID_LEN));

		$jsonString = substr($data, 1 + Constants::BALLOT_ID_LEN);

		// Verify json string first
		$decodeResult = json_decode($jsonString, true);
		if (!is_array($decodeResult)) {
			throw new BadMessageException();
		}

		$values = AssocArray::byAssocArray($decodeResult, ['d', 's', 'a', 't', 'o', 'c']);
		if (null === $values) {
			throw new BadMessageException();
		}

		// build choices
		$choices = [];
		foreach ($values->getValue('c') as $choice) {
			$choice = AssocArray::byAssocArray($choice, ['i', 'n', 'o']);
			if (null === $choice) {
				throw new BadMessageException();
			}

			$choices[] = new BallotChoice(
				$choice->getValue('i'),
				$choice->getValue('n'),
				$choice->getValue('o'),
				$choice->getValue('r', []),
				$choice->getValue('t')
			);
		}

		return new BallotCreateMessage(
			$ballotId,
			$values->getValue('d'),
			State::from($values->getValue('s')),
			VotingMode::from($values->getValue('a')),
			ResultsDisclosureType::from($values->getValue('t')),
			DisplayMode::from($values->getValue('u') ?? DisplayMode::LIST->value),
			$choices,
			$values->getValue('p', [])
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractGroupId(Closure $piece): GroupId
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function extractJson(string $data): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::BALLOT_ID_LEN
			// A {} string?
			+ 2) {
			throw new BadMessageException();
		}
	}
}
