<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Exceptions\InvalidArgumentException;
use Threema\MsgApi\Messages\BallotVoteMessage;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\Voting\VoteChoice;

class BallotVoteMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws BadMessageException
	 * @throws InvalidArgumentException
	 */
	public static function deserialize(string $data, Closure $piece): BallotVoteMessage
	{
		$creator = $piece->__invoke(Constants::IDENTITY_LEN);
		$ballotId = new BallotId($piece->__invoke(Constants::BALLOT_ID_LEN));

		$jsonString = substr($data, 1 + Constants::IDENTITY_LEN + Constants::BALLOT_ID_LEN);

		// Verify json string first
		$decodeResult = json_decode($jsonString, false);
		if (!is_array($decodeResult)) {
			throw new BadMessageException();
		}

		$votes = [];

		foreach ($decodeResult as $voteData) {
			if (2 !== count($voteData)) {
				throw new BadMessageException();
			}

			$choiceId = $voteData[0] ?? null;
			$value = $voteData[1] ?? null;

			if (!is_int($choiceId)) {
				throw new BadMessageException();
			}

			if (!in_array($value, [0, 1, '0', '1', true, false])) {
				throw new BadMessageException();
			}
			$votes[] = new VoteChoice($choiceId, (bool)$value);
		}

		return new BallotVoteMessage(
			$creator,
			$ballotId,
			$votes
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractGroupId(Closure $piece): GroupId
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function extractJson(string $data): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::IDENTITY_LEN
			+ Constants::BALLOT_ID_LEN
			// A {} string?
			+ 2) {
			throw new BadMessageException();
		}
	}
}
