<?php
/**
 * PHP version 8.1
 *
 * @category  Interface
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Messages\ThreemaMessage;
use Threema\MsgApi\Types\GroupId;

interface CustomMessageSerializer
{
	/**
	 * Serialize a given ThreemaMessage into a message data string.
	 */
	public static function serialize(): string;

	/**
	 * Deserialize a message string into the proper ThreemaMessage type.
	 */
	public static function deserialize(string $data, Closure $piece): ThreemaMessage;

	/**
	 * Extract the group identifier from the given message data string.
	 */
	public static function extractGroupId(Closure $piece): GroupId;

	/**
	 * Extract the json payload from the given message data string.
	 */
	public static function extractJson(string $data): string;

	/**
	 * Validate the given message data string.
	 * @throws BadMessageException
	 */
	public static function validate(string $data, int $realDataLength): void;
}
