<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Core\AssocArray;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Messages\GroupBallotCreateMessage;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\Voting\BallotChoice;
use Threema\MsgApi\Types\Voting\DisplayMode;
use Threema\MsgApi\Types\Voting\ResultsDisclosureType;
use Threema\MsgApi\Types\Voting\State;
use Threema\MsgApi\Types\Voting\VotingMode;

class GroupBallotCreateMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function deserialize(string $data, Closure $piece): GroupBallotCreateMessage
	{
		$groupId = self::extractGroupId($piece);
		$ballotId = new BallotId($piece->__invoke(Constants::BALLOT_ID_LEN));

		$jsonString = self::extractJson($data);

		// Verify json string first
		$decodeResult = json_decode($jsonString, true);
		if (!is_array($decodeResult)) {
			throw new BadMessageException();
		}

		$values = AssocArray::byAssocArray($decodeResult, ['d', 's', 'a', 't', 'c']);
		if (null === $values) {
			throw new BadMessageException();
		}

		// build choices
		$choices = [];
		foreach ($values->getValue('c') as $choice) {
			$choice = AssocArray::byAssocArray($choice, ['i', 'n', 'o']);
			if (null === $choice) {
				throw new BadMessageException();
			}

			$choices[] = new BallotChoice(
				$choice->getValue('i'),
				$choice->getValue('n'),
				$choice->getValue('o'),
				$choice->getValue('r', []),
				$choice->getValue('t')
			);
		}

		return new GroupBallotCreateMessage(
			$groupId,
			$ballotId,
			$values->getValue('d'),
			State::tryFrom($values->getValue('s')),
			VotingMode::tryFrom($values->getValue('a')),
			ResultsDisclosureType::tryFrom($values->getValue('t')),
			DisplayMode::tryFrom($values->getValue('u') ?? DisplayMode::LIST->value),
			$choices,
			$values->getValue('p', [])
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractGroupId(Closure $piece): GroupId
	{
		return new GroupId(
			$piece->__invoke(Constants::IDENTITY_LEN),
			$piece->__invoke(Constants::GROUP_ID_LEN)
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractJson(string $data): string
	{
		return substr($data, 1 + Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN + Constants::BALLOT_ID_LEN);
	}

	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::IDENTITY_LEN
			+ Constants::GROUP_ID_LEN
			+ Constants::BALLOT_ID_LEN
			// A {} string?
			+ 2) {
			throw new BadMessageException();
		}
	}
}
