<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Messages\GroupBallotVoteMessage;
use Threema\MsgApi\Types\BallotId;
use Threema\MsgApi\Types\GroupId;
use Threema\MsgApi\Types\Voting\VoteChoice;

class GroupBallotVoteMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function deserialize(string $data, Closure $piece): GroupBallotVoteMessage
	{
		$groupId = self::extractGroupId($piece);
		$creator = $piece->__invoke(Constants::IDENTITY_LEN);
		$ballotId = new BallotId($piece->__invoke(Constants::BALLOT_ID_LEN));
		$jsonString = self::extractJson($data);

		// Verify json string first
		$decodeResult = json_decode($jsonString, false);
		if (!is_array($decodeResult)) {
			throw new BadMessageException();
		}

		$votes = [];

		foreach ($decodeResult as $voteData) {
			if (2 !== count($voteData)) {
				throw new BadMessageException();
			}

			$choiceId = $voteData[0] ?? null;
			$value = $voteData[1] ?? null;

			if (!is_int($choiceId)) {
				throw new BadMessageException();
			}

			if (!in_array($value, [0, 1, '0', '1', true, false])) {
				throw new BadMessageException();
			}

			$votes[] = new VoteChoice($choiceId, (bool)$value);
		}

		return new GroupBallotVoteMessage($groupId, $creator, $ballotId, $votes);
	}

	public static function extractGroupId(Closure $piece): GroupId
	{
		return new GroupId(
			$piece->__invoke(Constants::IDENTITY_LEN),
			$piece->__invoke(Constants::GROUP_ID_LEN)
		);
	}

	public static function extractJson(string $data): string
	{
		return substr($data, 1 + Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN + Constants::IDENTITY_LEN + Constants::BALLOT_ID_LEN);
	}

	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::IDENTITY_LEN
			+ Constants::GROUP_ID_LEN
			+ Constants::IDENTITY_LEN
			+ Constants::BALLOT_ID_LEN
			// A {} string?
			+ 2) {
			throw new BadMessageException();
		}
	}
}
