<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Core\AssocArray;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Messages\GroupFileMessage;
use Threema\MsgApi\Types\FileRenderingType;
use Threema\MsgApi\Types\GroupId;

class GroupFileMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function deserialize(string $data, Closure $piece): GroupFileMessage
	{
		$groupId = self::extractGroupId($piece);

		$jsonString = self::extractJson($data);

		// Verify json string first
		$decodeResult = json_decode($jsonString, true);
		if (null === $decodeResult || false === $decodeResult) {
			throw new BadMessageException();
		}

		$values = AssocArray::byAssocArray($decodeResult, ['b', 'k', 'm', 's']);
		if (null === $values) {
			throw new BadMessageException();
		}

		return new GroupFileMessage(
			$groupId,
			$values->getValue('m'),
			$values->getValue('s'),
			$values->getValue('b'),
			$values->getValue('k'),
			$values->getValue('t'),
			$values->getValue('p'),
			$values->getValue('d'),
			FileRenderingType::tryFrom($values->getValue('j')),
			$values->getValue('c'),
			$values->getValue('x'),
			$values->getValue('n')
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractGroupId(Closure $piece): GroupId
	{
		return new GroupId(
			$piece->__invoke(Constants::IDENTITY_LEN),
			$piece->__invoke(Constants::GROUP_ID_LEN)
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractJson(string $data): string
	{
		return substr($data, 1 + Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN);
	}

	/**
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN
			// A {} string?
			+ 2) {
			throw new BadMessageException();
		}
	}
}
