<?php
/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Serializers;

use Closure;
use Exception;
use Threema\MsgApi\Constants;
use Threema\MsgApi\Exceptions\BadMessageException;
use Threema\MsgApi\Messages\GroupLocationMessage;
use Threema\MsgApi\Types\GroupId;

class GroupLocationMessageSerializer implements CustomMessageSerializer
{
	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function serialize(): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws Exception
	 */
	public static function deserialize(string $data, Closure $piece): GroupLocationMessage
	{
		$groupId = self::extractGroupId($piece);
		$locationStrings = explode("\n", substr($data, Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN + 1));
		$pieces = explode(',', $locationStrings[0]);

		$latitude = doubleval($pieces[0]);
		$longitude = doubleval($pieces[1]);
		$accuracy = 0;
		if (count($pieces) >= 3) {
			$accuracy = doubleval($pieces[2]);
		}

		$poiName = null;
		$poiAddress = null;

		if (count($locationStrings) >= 2) {
			$poiName = $locationStrings[1];

			if (count($locationStrings) >= 3) {
				$poiAddress = str_replace('\n', "\n", $locationStrings[2]);
			}
		}

		return new GroupLocationMessage(
			$groupId,
			$latitude,
			$longitude,
			$poiAddress,
			$accuracy,
			$poiName
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractGroupId(Closure $piece): GroupId
	{
		return new GroupId(
			$piece->__invoke(Constants::IDENTITY_LEN),
			$piece->__invoke(Constants::GROUP_ID_LEN)
		);
	}

	/**
	 * @throws Exception
	 */
	public static function extractJson(string $data): string
	{
		throw new Exception('Not implemented.');
	}

	/**
	 * @throws BadMessageException
	 * @throws Exception
	 */
	public static function validate(string $data, int $realDataLength): void
	{
		if ($realDataLength < 1
			+ Constants::IDENTITY_LEN + Constants::GROUP_ID_LEN
			+ 3) {
			throw new BadMessageException();
		}
	}
}
