<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Tools;

class FileAnalysisTool
{
	/**
	 * @param bool $readExifTags (only needed in ImageMessage receives)
	 */
	public function analyse(string $file, bool $readExifTags = false): ?FileAnalysisResult
	{
		//check if file exists
		if (false === file_exists($file)) {
			return null;
		}

		//is not a file
		if (false === is_file($file)) {
			return null;
		}

		//get file size
		$size = filesize($file);

		$mimeType = '';
		//mime type getter
		if (function_exists('finfo_open')) {
			$finfo = finfo_open(FILEINFO_MIME_TYPE);
			$mimeType = finfo_file($finfo, $file);
		} elseif (function_exists('mime_content_type')) {
			$mimeType = mime_content_type($file);
		}

		//default mime type
		if (strlen((string)$mimeType) == 0) {
			//default mime type
			$mimeType = 'application/octet-stream';
		}

		// try to get caption
		$exifTags = [];
		if (true === $readExifTags && function_exists('exif_read_data')) {
			$exifTags = exif_read_data($file);
		}

		return new FileAnalysisResult($mimeType, $size, $file, $exifTags);
	}

	public function getContent($file): bool|string
	{
		return file_get_contents($file);
	}

	/**
	 * @return bool
	 */
	public function isImage(string $file)
	{
		$r = $this->analyse($file);

		return null !== $r && in_array($r->getMimeType(), [
			'image/jpg',
			'image/jpeg',
			'image/png'
		]);
	}
}
