<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Tools;

use Threema\MsgApi\Core\Exception;

class ThreemaIdTool
{
	public static function normalizeThreemaId(string $threemaId): string
	{
		return trim(strtoupper($threemaId));
	}

	/**
	 * @note ThreemaId should be normalized (trimmed, 8 chars, all uppercase)
	 * @throws \Threema\MsgApi\Core\Exception
	 */
	public static function verifyThreemaId(string $threemaId): string
	{
		if (8 !== strlen($threemaId) || strtoupper($threemaId) !== $threemaId) {
			throw new Exception("Invalid threema id <{$threemaId}>");
		}

		return $threemaId;
	}

	/**
	 * @throws \Threema\MsgApi\Core\Exception
	 */
	public static function toThreemaId(string $input): string
	{
		$normalized = self::normalizeThreemaId($input);

		return self::verifyThreemaId($normalized);
	}
}
