<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi\Types;

use Stringable;
use Threema\MsgApi\Exceptions\InvalidArgumentException;

class BallotId implements Stringable
{
	private ?string $ballotId;

	/**
	 * @throws InvalidArgumentException
	 */
	public function __construct(string $ballotId)
	{
		$this->ballotId = null;
		$this->setBallotId($ballotId);
	}

	public function __serialize(): array
	{
		return [
			'ballotId' => bin2hex($this->ballotId)
		];
	}

	public function __unserialize(array $data): void
	{
		$this->ballotId = hex2bin((string)$data['ballotId']);
	}

	public function __toString()
	{
		return 'BallotId{ballotId=' . $this->ballotId . '}';
	}

	public function getBallotId(): ?string
	{
		return $this->ballotId;
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function setBallotId(string $ballotId): BallotId
	{
		if (null !== $this->ballotId) {
			throw new InvalidArgumentException('ballot id already defined');
		}

		if (strlen($ballotId) !== 8) {
			throw new InvalidArgumentException('ballot id must be an 8 byte string');
		}

		$this->ballotId = $ballotId;

		return $this;
	}
}
