<?php
/**
 * BallotChoice
 *
 * PHP version 8.1
 *
 * @category  Class
 * @author    Threema Gateway team
 * @copyright Copyright (c) 2023 Threema GmbH
 * @see       https://gateway.threema.ch/en/developer
 */

/**
 * Threema Gateway Php SDK
 *
 * This SDK allows for preparing, sending and receiving of Threema Messages via Threema Gateway.
 *
 * The version of the model input document: 1
 * Generated by: https://gateway.threema.ch/en/developer
 * MsgApi SDK Generator version: 0.1.0
 */

namespace Threema\MsgApi\Types\Voting;

use JsonSerializable;
use Stringable;
use Threema\MsgApi\Tools\Util;

/**
 * BallotChoice Class Doc Comment
 *
 * @category Class
 * @description A poll choice item that is part of a poll message.
 * @author   Threema Gateway team
 * @see     https://gateway.threema.ch/en/developer
 */
class BallotChoice implements JsonSerializable, Stringable
{
	public function __construct(
		private readonly int $identifier,
		private readonly string $name,
		private readonly int $order,
		private readonly ?array $result = null,
		private readonly ?int $totalVotes = null
	) {
	}

	public function __toArray(): array
	{
		return array_filter([
			'i' => $this->getIdentifier(),
			'n' => $this->getName(),
			'o' => $this->getOrder(),
			'r' => $this->getResult(),
			't' => $this->getTotalVotes()
		], fn ($v) => Util::array_filter_callback($v));
	}

	/**
	 * Gets the string presentation of the object
	 */
	public function __toString(): string
	{
		return 'Ballot{' . 'identifier=' . $this->identifier . ', '
			. 'name=' . $this->name . ', '
			. 'order=' . $this->order . ', '
			. 'result=' . print_r($this->result, true) . ', '
			. 'totalVotes=' . $this->totalVotes . '}';
	}

	/**
	 * Data that can be natively serialized by `json_encode()`.
	 */
	public function jsonSerialize(): string|array
	{
		return $this->__toArray();
	}

	/**
	 * Show all the invalid properties with reasons.
	 *
	 * @return array invalid properties with reasons
	 */
	public function listInvalidProperties(): array
	{
		$invalidProperties = [];

		if (null === $this->identifier) {
			$invalidProperties[] = "'identifier' can't be null";
		}
		if (null === $this->name) {
			$invalidProperties[] = "'name' can't be null";
		}
		if (null === $this->order) {
			$invalidProperties[] = "'order' can't be null";
		}
//		if ($this->result === null) {
//			$invalidProperties[] = "'result' can't be null";
//		}
		return $invalidProperties;
	}

	/**
	 * Validate all the properties in the model
	 * return true if all passed
	 *
	 * @return bool True if all properties are valid
	 */
	public function valid(): bool
	{
		return count($this->listInvalidProperties()) === 0;
	}

	/**
	 * Gets identifier
	 */
	public function getIdentifier(): int
	{
		return $this->identifier;
	}

	/**
	 * Gets name
	 */
	public function getName(): string
	{
		return $this->name;
	}

	/**
	 * Gets order
	 *
	 * @deprecated
	 */
	public function getOrder(): int
	{
		return $this->order;
	}

	/**
	 * Gets result
	 */
	public function getResult(): ?array
	{
		return $this->result;
	}

	/**
	 * Gets totalVotes
	 */
	public function getTotalVotes(): ?int
	{
		return $this->totalVotes;
	}
}
