<?php
/**
 * FieldElement
 *
 * SplFixedArray with more functions.
 *
 * @author Devi Mandiri <devi.mandiri@gmail.com>
 * @see   https://github.com/devi/Salt
 */
class FieldElement extends SplFixedArray
{
	public function toString(): string
	{
		$buf = '';
		foreach ($this->toArray() as $c) {
			$buf .= chr($c);
		}

		return $buf;
	}

	public function toHex(): string
	{
		$hextable = '0123456789abcdef';
		$buf = '';
		foreach ($this->toArray() as $c) {
			$buf .= $hextable[$c >> 4];
			$buf .= $hextable[$c & 0x0f];
		}

		return $buf;
	}

	public function toBase64(): string
	{
		return base64_encode($this->toString());
	}

	/**
	 * @return false|string
	 */
	public function toJson()
	{
		return json_encode($this->toString());
	}

	public function slice($offset, $length = null): self
	{
		$length = $length ? $length : $this->getSize() - $offset;
		$slice = new FieldElement($length);
		for ($i = 0; $i < $length; ++$i) {
			$slice[$i] = $this->offsetGet($i + $offset);
		}

		return $slice;
	}

	public function copy($src, $size, $offset = 0, $srcOffset = 0): void
	{
		for ($i = 0; $i < $size; ++$i) {
			$this->offsetSet($i + $offset, $src[$i + $srcOffset]);
		}
	}

	public static function fromArray($array, $save_indexes = true): self
	{
		$l = count($array);
		$fe = new FieldElement($l);
		$array = $save_indexes ? $array : array_values($array);
		foreach ($array as $k => $v) {
			$fe[$k] = $v;
		}

		return $fe;
	}

	public static function fromString($str): self
	{
		return static::fromArray(unpack('C*', (string)$str), false);
	}

	public static function fromHex($hex): self
	{
		$hex = preg_replace('/[^0-9a-f]/', '', (string)$hex);

		return static::fromString(pack('H*', $hex));
	}

	public static function fromBase64($base64): self
	{
		return FieldElement::fromString(base64_decode((string)$base64, true));
	}

	public static function fromJson($json): self
	{
		return FieldElement::fromArray(json_decode((string)$json, true));
	}
}
